@@ -15,16 +15,17 @@ class Entry:
 
     def parse_tags(self):
         fulltext = " ".join([self.title, self.body]).lower()
-        tags = re.findall(r"([%s]\w+)" % self.journal.config['tagsymbols'], fulltext)
-        self.tags = set(tags)
+        tags = re.findall(r'(?u)([{tags}]\w+)'.format(tags=self.journal.config['tagsymbols']), fulltext, re.UNICODE)
+        self.tags = tags
+        return set(tags)
 
-    def __str__(self):
+    def __unicode__(self):
         """Returns a string representation of the entry to be written into a journal file."""
         date_str = self.date.strftime(self.journal.config['timeformat'])
         title = date_str + " " + self.title
         body = self.body.strip()
 
-        return "{title}{sep}{body}\n".format(
+        return u"{title}{sep}{body}\n".format(
             title=title,
             sep="\n" if self.body else "",
             body=body
@@ -50,14 +51,14 @@ class Entry:
         # Suppress bodies that are just blanks and new lines.
         has_body = len(self.body) > 20 or not all(char in (" ", "\n") for char in self.body)
 
-        return "{title}{sep}{body}\n".format(
+        return u"{title}{sep}{body}\n".format(
             title=title,
             sep="\n" if has_body else "",
             body=body if has_body else "",
         )
 
     def __repr__(self):
-        return str(self)
+        return "<Entry '{0}' on {1}>".format(self.title.strip(), self.date.strftime("%Y-%m-%d %H:%M"))
 
     def to_dict(self):
         return {
@@ -74,7 +75,7 @@ class Entry:
         space = "\n"
         md_head = "###"
 
-        return "{md} {date}, {title} {body} {space}".format(
+        return u"{md} {date}, {title} {body} {space}".format(
             md=md_head,
             date=date_str,
             title=self.title,
