@@ -7,9 +7,16 @@
     license: MIT, see LICENSE for more details.
 """
 
-import Journal
-import exporters
-import install
+try:
+    from . import Journal
+    from . import util
+    from . import exporters
+    from . import install
+except (SystemError, ValueError):
+    import Journal
+    import util
+    import exporters
+    import install
 import os
 import tempfile
 import subprocess
@@ -18,16 +25,12 @@ import sys
 try: import simplejson as json
 except ImportError: import json
 
-
-__title__ = 'jrnl'
-__version__ = '1.0.0-rc1'
-__author__ = 'Manuel Ebert, Stephan Gabler'
-__license__ = 'MIT'
-
-CONFIG_PATH = os.path.expanduser('~/.jrnl_config')
+xdg_config = os.environ.get('XDG_CONFIG_HOME')
+CONFIG_PATH = os.path.join(xdg_config, "jrnl") if xdg_config else os.path.expanduser('~/.jrnl_config')
 PYCRYPTO = install.module_exists("Crypto")
 
-def parse_args():
+
+def parse_args(args=None):
     parser = argparse.ArgumentParser()
     composing = parser.add_argument_group('Composing', 'Will make an entry out of whatever follows as arguments')
     composing.add_argument('-date', dest='date', help='Date, e.g. "yesterday at 5pm"')
@@ -36,26 +39,26 @@ def parse_args():
 
     reading = parser.add_argument_group('Reading', 'Specifying either of these parameters will display posts of your journal')
     reading.add_argument('-from', dest='start_date', metavar="DATE", help='View entries after this date')
-    reading.add_argument('-to', dest='end_date', metavar="DATE", help='View entries before this date')
+    reading.add_argument('-until', '-to', dest='end_date', metavar="DATE", help='View entries before this date')
     reading.add_argument('-and', dest='strict', action="store_true", help='Filter by tags using AND (default: OR)')
     reading.add_argument('-n', dest='limit', default=None, metavar="N", help='Shows the last n entries matching the filter', nargs="?", type=int)
     reading.add_argument('-short', dest='short', action="store_true", help='Show only titles or line containing the search tags')
 
     exporting = parser.add_argument_group('Export / Import', 'Options for transmogrifying your journal')
     exporting.add_argument('--tags', dest='tags', action="store_true", help='Returns a list of all tags and number of occurences')
-    exporting.add_argument('--json', dest='json', action="store_true", help='Returns a JSON-encoded version of the Journal')
-    exporting.add_argument('--markdown', dest='markdown', action="store_true", help='Returns a Markdown-formated version of the Journal')
+    exporting.add_argument('--export', metavar='TYPE', dest='export', help='Export your journal to Markdown, JSON or Text', nargs='?', default=False, const=None)
+    exporting.add_argument('-o', metavar='OUTPUT', dest='output', help='The output of the file can be provided when using with --export', nargs='?', default=False, const=None)
     exporting.add_argument('--encrypt',  metavar='FILENAME', dest='encrypt', help='Encrypts your existing journal with a new password', nargs='?', default=False, const=None)
     exporting.add_argument('--decrypt',  metavar='FILENAME', dest='decrypt', help='Decrypts your journal and stores it in plain text', nargs='?', default=False, const=None)
     exporting.add_argument('--delete-last', dest='delete_last', help='Deletes the last entry from your journal file.', action="store_true")
 
-    return parser.parse_args()
+    return parser.parse_args(args)
 
 def guess_mode(args, config):
     """Guesses the mode (compose, read or export) from the given arguments"""
     compose = True
     export = False
-    if args.json or args.decrypt is not False or args.encrypt is not False or args.markdown or args.tags or args.delete_last:
+    if args.decrypt is not False or args.encrypt is not False or args.export is not False or args.tags or args.delete_last:
         compose = False
         export = True
     elif args.start_date or args.end_date or args.limit or args.strict or args.short:
@@ -75,7 +78,7 @@ def get_text_from_editor(config):
             raw = f.read()
         os.remove(tmpfile)
     else:
-        print('[Nothing saved to file]')
+        util.prompt('[Nothing saved to file]')
         raw = ''
 
     return raw
@@ -87,36 +90,19 @@ def encrypt(journal, filename=None):
     journal.make_key(prompt="Enter new password:")
     journal.config['encrypt'] = True
     journal.write(filename)
-    print("Journal encrypted to {}.".format(filename or journal.config['journal']))
+    util.prompt("Journal encrypted to {0}.".format(filename or journal.config['journal']))
 
 def decrypt(journal, filename=None):
     """ Decrypts into new file. If filename is not set, we encrypt the journal file itself. """
     journal.config['encrypt'] = False
     journal.config['password'] = ""
     journal.write(filename)
-    print("Journal decrypted to {}.".format(filename or journal.config['journal']))
-
-def print_tags(journal):
-        """Prints a list of all tags and the number of occurances."""
-        # Astute reader: should the following line leave you as puzzled as me the first time
-        # I came across this construction, worry not and embrace the ensuing moment of enlightment.
-        tags = [tag
-            for entry in journal.entries
-            for tag in set(entry.tags)
-        ]
-        # To be read: [for entry in journal.entries: for tag in set(entry.tags): tag]
-        tag_counts = {(tags.count(tag), tag) for tag in tags}
-        if min(tag_counts)[0] == 0:
-            tag_counts = filter(lambda x: x[0] > 1, tag_counts)
-            print('[Removed tags that appear only once.]')
-        for n, tag in sorted(tag_counts, reverse=False):
-            print("{:20} : {}".format(tag, n))
-
+    util.prompt("Journal decrypted to {0}.".format(filename or journal.config['journal']))
 
 def touch_journal(filename):
     """If filename does not exist, touch the file"""
     if not os.path.exists(filename):
-        print("[Journal created at {}]".format(filename))
+        util.prompt("[Journal created at {0}]".format(filename))
         open(filename, 'a').close()
 
 def update_config(config, new_config, scope):
@@ -129,21 +115,26 @@ def update_config(config, new_config, scope):
         config.update(new_config)
 
 
-def cli():
+def cli(manual_args=None):
     if not os.path.exists(CONFIG_PATH):
         config = install.install_jrnl(CONFIG_PATH)
     else:
         with open(CONFIG_PATH) as f:
-            config = json.load(f)
+            try:
+                config = json.load(f)
+            except ValueError as e:
+                util.prompt("[There seems to be something wrong with your jrnl config at {0}: {1}]".format(CONFIG_PATH, e.message))
+                util.prompt("[Entry was NOT added to your journal]")
+                sys.exit(1)
         install.update_config(config, config_path=CONFIG_PATH)
 
     original_config = config.copy()
     # check if the configuration is supported by available modules
     if config['encrypt'] and not PYCRYPTO:
-        print("According to your jrnl_conf, your journal is encrypted, however PyCrypto was not found. To open your journal, install the PyCrypto package from http://www.pycrypto.org.")
-        sys.exit(-1)
+        util.prompt("According to your jrnl_conf, your journal is encrypted, however PyCrypto was not found. To open your journal, install the PyCrypto package from http://www.pycrypto.org.")
+        sys.exit(1)
 
-    args = parse_args()
+    args = parse_args(manual_args)
 
     # If the first textual argument points to a journal file,
     # use this!
@@ -159,17 +150,21 @@ def cli():
     mode_compose, mode_export = guess_mode(args, config)
 
     # open journal file or folder
-    if os.path.isdir(config['journal']) and config['journal'].endswith(".dayone"):
-        journal = Journal.DayOne(**config)
+    if os.path.isdir(config['journal']):
+        if config['journal'].strip("/").endswith(".dayone") or \
+           "entries" in os.listdir(config['journal']):
+            journal = Journal.DayOne(**config)
+        else:
+            util.prompt("[Error: {0} is a directory, but doesn't seem to be a DayOne journal either.".format(config['journal']))
+            sys.exit(1)
     else:
         journal = Journal.Journal(**config)
 
-
     if mode_compose and not args.text:
         if config['editor']:
             raw = get_text_from_editor(config)
         else:
-            raw = raw_input("[Compose Entry] ")
+            raw = util.py23_input("[Compose Entry] ")
         if raw:
             args.text = [raw]
         else:
@@ -178,32 +173,32 @@ def cli():
     # Writing mode
     if mode_compose:
         raw = " ".join(args.text).strip()
+        if util.PY2 and type(raw) is not unicode:
+            raw = raw.decode(sys.getfilesystemencoding())
         entry = journal.new_entry(raw, args.date)
         entry.starred = args.star
-        print("[Entry added to {} journal]").format(journal_name)
+        util.prompt("[Entry added to {0} journal]".format(journal_name))
         journal.write()
-
-    # Reading mode
-    elif not mode_export:
+    else:
         journal.filter(tags=args.text,
                        start_date=args.start_date, end_date=args.end_date,
                        strict=args.strict,
                        short=args.short)
         journal.limit(args.limit)
-        print(journal)
+
+    # Reading mode
+    if not mode_compose and not mode_export:
+        print(journal.pprint())
 
     # Various export modes
     elif args.tags:
-        print_tags(journal)
+        print(exporters.to_tag_list(journal))
 
-    elif args.json: # export to json
-        print(exporters.to_json(journal))
-
-    elif args.markdown: # export to json
-        print(exporters.to_md(journal))
+    elif args.export is not False:
+        print(exporters.export(journal, args.export, args.output))
 
     elif (args.encrypt is not False or args.decrypt is not False) and not PYCRYPTO:
-        print("PyCrypto not found. To encrypt or decrypt your journal, install the PyCrypto package from http://www.pycrypto.org.")
+        util.prompt("PyCrypto not found. To encrypt or decrypt your journal, install the PyCrypto package from http://www.pycrypto.org.")
 
     elif args.encrypt is not False:
         encrypt(journal, filename=args.encrypt)
@@ -221,7 +216,7 @@ def cli():
 
     elif args.delete_last:
         last_entry = journal.entries.pop()
-        print("[Deleted Entry:]")
+        util.prompt("[Deleted Entry:]")
         print(last_entry)
         journal.write()
 
